@@ -1,4 +1,4 @@
-from __future__ import division
+
 import pymc as pm
 import numpy as np
 from scipy import stats
@@ -44,13 +44,33 @@ def generate_wfpt_stochastic_class(wiener_params=None, sampling_method='cdf', cd
         wiener_params = {'err': 1e-4, 'n_st':2, 'n_sz':2,
                          'use_adaptive':1,
                          'simps_err':1e-3,
-                         'w_outlier': 0.1}
+                         'w_outlier': 0.1,
+                         }
     wp = wiener_params
 
     #create likelihood function
     def wfpt_like(x, v, sv, a, z, sz, t, st, p_outlier=0):
-        return hddm.wfpt.wiener_like(x['rt'].values, v, sv, a, z, sz, t, st, p_outlier=p_outlier, **wp)
+        if x['rt'].abs().max() < 998:
+            return hddm.wfpt.wiener_like(x['rt'].values, v, sv, a, z, sz, t, st,
+                                         p_outlier=p_outlier, **wp)
+        else:  # for missing RTs. Currently undocumented.
+            noresponse = x['rt'].abs() >= 999
+            ## get sum of log p for trials with RTs as usual ##
+            logp_resp = hddm.wfpt.wiener_like(x.loc[~noresponse, 'rt'].values,
+                                             v, sv, a, z, sz, t, st, p_outlier=p_outlier, **wp)
 
+            # get number of no-response trials
+            n_noresponse = sum(noresponse)
+            k_upper = sum(x.loc[noresponse, 'rt'] > 0)
+
+            # percentage correct according to probability to get to upper boundary
+            if v == 0:
+                p_upper = z
+            else:
+                p_upper = (np.exp(-2 * a * z * v) - 1) / (np.exp(-2 * a * v) - 1)
+
+            logp_noresp = stats.binom.logpmf(k_upper, n_noresponse, p_upper)
+            return logp_resp + logp_noresp
 
     #create random function
     def random(self):
@@ -75,7 +95,7 @@ def generate_wfpt_stochastic_class(wiener_params=None, sampling_method='cdf', cd
 
     #add pdf and cdf_vec to the class
     wfpt.pdf = pdf
-    wfpt.cdf_vec = lambda self: hddm.wfpt.gen_cdf_using_pdf(time=cdf_range[1], **dict(self.parents.items() + wp.items()))
+    wfpt.cdf_vec = lambda self: hddm.wfpt.gen_cdf_using_pdf(time=cdf_range[1], **dict(list(self.parents.items()) + list(wp.items())))
     wfpt.cdf = cdf
     wfpt.random = random
 
@@ -111,26 +131,49 @@ def add_quantiles_functions_to_pymc_class(pymc_class):
         self._quantiles_edges = np.asarray(quantiles)
 
         data = self.value
+        
+        if np.all(~np.isnan(data['rt'])):
+            
+            #get proportion of data fall between the quantiles
+            quantiles = np.array(quantiles)
+            pos_proportion = np.diff(np.concatenate((np.array([0.]), quantiles, np.array([1.]))))
+            neg_proportion = pos_proportion[::-1]
+            proportion = np.concatenate((neg_proportion[::-1],  pos_proportion))
+            self._n_samples = len(data)
 
-        #get proportion of data fall between the quantiles
-        quantiles = np.array(quantiles)
-        pos_proportion = np.diff(np.concatenate((np.array([0.]), quantiles, np.array([1.]))))
-        neg_proportion = pos_proportion[::-1]
-        proportion = np.concatenate((neg_proportion[::-1],  pos_proportion))
-        self._n_samples = len(data)
-
-        # extract empirical RT at the quantiles
-        self._empirical_quantiles = hddm.utils.data_quantiles(data, quantiles)
-        ub_emp_rt = self._empirical_quantiles[1]
-        lb_emp_rt = -self._empirical_quantiles[0]
-        self._emp_rt = np.concatenate((lb_emp_rt[::-1], np.array([0.]), ub_emp_rt))
-
-        #get frequency of observed values
-        freq_obs = np.zeros(len(proportion))
-        freq_obs[:len(quantiles)+1] = sum(data.rt<0) * neg_proportion
-        freq_obs[len(quantiles)+1:] = sum(data.rt>0) * pos_proportion
-        self._freq_obs = freq_obs
+            # extract empirical RT at the quantiles
+            self._empirical_quantiles = hddm.utils.data_quantiles(data, quantiles)
+            ub_emp_rt = self._empirical_quantiles[1]
+            lb_emp_rt = -self._empirical_quantiles[0]
+            self._emp_rt = np.concatenate((lb_emp_rt[::-1], np.array([0.]), ub_emp_rt))
 
+            #get frequency of observed values
+            freq_obs = np.zeros(len(proportion))
+            freq_obs[:len(quantiles)+1] = sum(data.rt<0) * neg_proportion
+            freq_obs[len(quantiles)+1:] = sum(data.rt>0) * pos_proportion
+            self._freq_obs = freq_obs
+        
+        else:
+            
+            #get proportion of data fall between the quantiles
+            quantiles = np.array(quantiles)
+            pos_proportion = np.diff(np.concatenate((np.array([0.]), quantiles, np.array([1.]))))
+            neg_proportion = np.array([1])
+            proportion = np.concatenate((neg_proportion[::-1],  pos_proportion))
+            self._n_samples = len(data)
+            
+            # extract empirical RT at the quantiles
+            self._empirical_quantiles = hddm.utils.data_quantiles(data, quantiles)
+            ub_emp_rt = self._empirical_quantiles[1]
+            lb_emp_rt = -self._empirical_quantiles[0]
+            self._emp_rt = np.concatenate((np.array([0.]), ub_emp_rt))
+            
+            #get frequency of observed values
+            freq_obs = np.zeros(len(proportion))
+            freq_obs[0] = sum(np.isnan(data.rt)) * neg_proportion
+            freq_obs[1:] = sum(data.rt>0) * pos_proportion
+            self._freq_obs = freq_obs
+        
     def set_quantiles_stats(self, quantiles, n_samples, emp_rt, freq_obs, p_upper):
         """
         set quantiles statistics (used when one do not to compute the statistics from the stochastic's value)
